%%
% This is a minimmal working example of the method presented in

% M. Vestner, R. Litman, E. Rodola, A. Bronstein, D. Cremers:
% Product Manifold Filter:
% Non-Rigid Shape Correspondence via
% Kernel Density Estimation in the Product Space
% IEEE Conference on Computer Vision and Pattern Recognition (CVPR), 2017

% from a possibly sparse and noise input matching between two
% three dimensional shapes a dense, bijective and accurate matching
% is derived by solving a sequence of linear assignment problems

% in this minimal working example the input correspondence is given by a
% sparse set of ground truth matchings.

% Last modified on 07/14/2017

%%
clear,clc
%%
addpath('fastAuction_v2.5')
%% Loading shapes
X = loadshape_minimal('data/0001.isometry.1.off');
Y = loadshape_minimal('data/0001.isometry.2.off');

matches_gt = get_gt(X,Y);

%% Parameters

nsparse = 20; %number of sparse inputs
niters = 3; %number of iterations


varx = 2*X.area; % variance of Gaussian on source
vary = 2*Y.area; % variance of Gaussian on target



%% In this scenario we initialize with sparse gt-matches
%% Replace the following lines to use a (sparse) matchings from your pipeline
xin = X.fps(1:nsparse);
yin = matches_gt(xin);

%%

for i=1:niters
    F = kernel_density_estimation(X,Y,xin,yin,varx,vary);
    xin = 1:X.n;
    yin = sparseAssignmentProblemAuctionAlgorithm(F,[],[],0);
    figure,vis_3D_correspondence(X,Y,yin);
    figure(444),plot_err_curve(xin,yin,matches_gt,Y.Gamma);hold on
end
%%